#import "ViewController.h"

#import "vtkCubeSource.h"

#import "vesActor.h"

#import "mvesPolydataMapper.h"
#import "mvesMaterials.h"
#import "mvesRenderer.h"
#import "mvesView.h"


@interface ViewController () {
@private
    vtkSmartPointer<vtkCubeSource> cubeSource;
    vesSharedPtr<mvesPolyDataMapper> mapper;
    vesSharedPtr<vesMaterial> material;
    vesSharedPtr<vesActor> actor;
}

@property (strong, nonatomic) IBOutlet mvesView *mView;

@end


@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    // construct
    cubeSource = vtkSmartPointer<vtkCubeSource>::New();
    mapper = vesSharedPtr<mvesPolyDataMapper>(new mvesPolyDataMapper());
    material = mvesMaterials::gouraudMaterial();
    actor = vesSharedPtr<vesActor>(new vesActor());
    
    // mapper
    mapper->setColor(1, 0, 0, 1);
    mapper->SetInputConnection(cubeSource->GetOutputPort());
    
    // actor
    actor->setMapper(mapper);
    actor->setMaterial(material);
    
    // renderer
    self.mView.renderer->addActor(actor);
    self.mView.renderer->resetCamera();

    // view
    [self.mView addDefaultGestureRecognizers];
}

@end
