#include "mvesShaderPrograms.h"

#include "vesShader.h"
#include "vesVertexAttributeKeys.h"
#include "vesVertexAttribute.h"
#include "vesBuiltinShaders.h"
#include "vesModelViewUniform.h"
#include "vesProjectionUniform.h"
#include "vesNormalMatrixUniform.h"


static vesSharedPtr<vesShaderProgram> newShaderProgram(std::string vs, std::string fs) {
    vesSharedPtr<vesShaderProgram> shaderProgram = vesSharedPtr<vesShaderProgram>(new vesShaderProgram());
    
    // uniform
    shaderProgram->addUniform(vesSharedPtr<vesUniform>(new vesModelViewUniform()));
    shaderProgram->addUniform(vesSharedPtr<vesUniform>(new vesProjectionUniform()));
    shaderProgram->addUniform(vesSharedPtr<vesUniform>(new vesNormalMatrixUniform()));
    
    // vertex attributes
    shaderProgram->addVertexAttribute(vesSharedPtr<vesVertexAttribute>(new vesPositionVertexAttribute()), vesVertexAttributeKeys::Position);
    shaderProgram->addVertexAttribute(vesSharedPtr<vesVertexAttribute>(new vesNormalVertexAttribute()), vesVertexAttributeKeys::Normal);
    shaderProgram->addVertexAttribute(vesSharedPtr<vesVertexAttribute>(new vesColorVertexAttribute()), vesVertexAttributeKeys::Color);
    shaderProgram->addVertexAttribute(vesSharedPtr<vesVertexAttribute>(new vesTextureCoordinateVertexAttribute()), vesVertexAttributeKeys::TextureCoordinate);
    
    // shaders
    shaderProgram->addShader(vesSharedPtr<vesShader>(new vesShader(vesShader::Vertex, vs)));
    shaderProgram->addShader(vesSharedPtr<vesShader>(new vesShader(vesShader::Fragment, fs)));
    
    return shaderProgram;
}

vesSharedPtr<vesShaderProgram> mvesShaderPrograms::gouraudShaderProgram() {
    return newShaderProgram(vesBuiltinShaders::vesShader_vert(), vesBuiltinShaders::vesShader_frag());
}

vesSharedPtr<vesShaderProgram> mvesShaderPrograms::gouraudTextureShaderProgram() {
    return newShaderProgram(vesBuiltinShaders::vesGouraudTexture_vert(), vesBuiltinShaders::vesGouraudTexture_frag());
}

vesSharedPtr<vesShaderProgram> mvesShaderPrograms::toonShaderProgram() {
    return newShaderProgram(vesBuiltinShaders::vesToonShader_vert(), vesBuiltinShaders::vesToonShader_frag());
}
